import uuid
from typing import List, Optional
from sqlalchemy.orm import Session
from sqlalchemy import or_

from models.fruit import Fruit
from schemas.fruit import FruitCreate, FruitUpdate

def get_all_fruits(db: Session) -> List[Fruit]:
    """
    Retrieves all fruits from the database.

    Args:
        db (Session): The database session.

    Returns:
        List[Fruit]: A list of all fruit objects.
    """
    return db.query(Fruit).all()

def get_fruit_by_id(db: Session, fruit_id: uuid.UUID) -> Optional[Fruit]:
    """
    Retrieves a single fruit by its ID.

    Args:
        db (Session): The database session.
        fruit_id (UUID): The ID of the fruit to retrieve.

    Returns:
        Optional[Fruit]: The fruit object if found, otherwise None.
    """
    return db.query(Fruit).filter(Fruit.id == fruit_id).first()

def get_fruits_by_name(db: Session, name: str) -> List[Fruit]:
    """
    Retrieves fruits by their name (case-insensitive).

    Args:
        db (Session): The database session.
        name (str): The name of the fruits to retrieve.

    Returns:
        List[Fruit]: A list of fruit objects matching the name.
    """
    return db.query(Fruit).filter(Fruit.name.ilike(f"%{name}%")).all()

def create_fruit(db: Session, fruit_data: FruitCreate) -> Fruit:
    """
    Creates a new fruit in the database.

    Args:
        db (Session): The database session.
        fruit_data (FruitCreate): The data for the fruit to create.

    Returns:
        Fruit: The newly created fruit object.
    """
    db_fruit = Fruit(**fruit_data.dict())
    db.add(db_fruit)
    db.commit()
    db.refresh(db_fruit)
    return db_fruit

def update_fruit(db: Session, fruit_id: uuid.UUID, fruit_data: FruitUpdate) -> Optional[Fruit]:
    """
    Updates an existing fruit in the database.

    Args:
        db (Session): The database session.
        fruit_id (UUID): The ID of the fruit to update.
        fruit_data (FruitUpdate): The updated data for the fruit.

    Returns:
        Optional[Fruit]: The updated fruit object if found, otherwise None.
    """
    db_fruit = db.query(Fruit).filter(Fruit.id == fruit_id).first()
    if not db_fruit:
        return None

    for field, value in fruit_data.dict(exclude_unset=True).items():
        setattr(db_fruit, field, value)

    db.commit()
    db.refresh(db_fruit)
    return db_fruit

def delete_fruit(db: Session, fruit_id: uuid.UUID) -> bool:
    """
    Deletes a fruit from the database.

    Args:
        db (Session): The database session.
        fruit_id (UUID): The ID of the fruit to delete.

    Returns:
        bool: True if the fruit was successfully deleted, False otherwise.
    """
    db_fruit = db.query(Fruit).filter(Fruit.id == fruit_id).first()
    if not db_fruit:
        return False

    db.delete(db_fruit)
    db.commit()
    return True

def search_fruits(db: Session, query: str) -> List[Fruit]:
    """
    Searches for fruits by name or description.

    Args:
        db (Session): The database session.
        query (str): The search query string.

    Returns:
        List[Fruit]: A list of fruit objects matching the search query.
    """
    return db.query(Fruit).filter(
        or_(Fruit.name.ilike(f"%{query}%"), Fruit.description.ilike(f"%{query}%"))
    ).all()